diff -ur x264.r439/common/common.c x264/common/common.c
--- x264.r439/common/common.c	2006-02-20 20:34:01.000000000 +0100
+++ x264/common/common.c	2006-03-01 14:49:30.000000000 +0100
@@ -62,6 +62,7 @@
     param->i_fps_num       = 25;
     param->i_fps_den       = 1;
     param->i_level_idc     = 51; /* as close to "unrestricted" as we can get */
+    param->i_slice_max     = 0;  /* don't restrict by default */
 
     /* Encoder parameters */
     param->i_frame_reference = 1;
diff -ur x264.r439/common/common.h x264/common/common.h
--- x264.r439/common/common.h	2006-02-20 20:34:01.000000000 +0100
+++ x264/common/common.h	2006-02-21 02:58:43.000000000 +0100
@@ -218,8 +218,7 @@
 */
 
 #define X264_BFRAME_MAX 16
-#define X264_SLICE_MAX 4
-#define X264_NAL_MAX (4 + X264_SLICE_MAX)
+#define X264_THREADS_MAX 4
 
 typedef struct x264_ratecontrol_t   x264_ratecontrol_t;
 typedef struct x264_vlc_table_t     x264_vlc_table_t;
@@ -229,13 +228,14 @@
     /* encoder parameters */
     x264_param_t    param;
 
-    x264_t *thread[X264_SLICE_MAX];
+    x264_t *thread[X264_THREADS_MAX];
 
     /* bitstream output */
     struct
     {
         int         i_nal;
-        x264_nal_t  nal[X264_NAL_MAX];
+        int         i_nal_bufsize;
+        x264_nal_t  * nal;
         int         i_bitstream;    /* size of p_bitstream */
         uint8_t     *p_bitstream;   /* will hold data for all nal */
         bs_t        bs;
diff -ur x264.r439/encoder/encoder.c x264/encoder/encoder.c
--- x264.r439/encoder/encoder.c	2006-02-20 20:34:01.000000000 +0100
+++ x264/encoder/encoder.c	2006-03-01 17:42:07.000000000 +0100
@@ -341,7 +341,7 @@
         return -1;
     }
 
-    h->param.i_threads = x264_clip3( h->param.i_threads, 1, X264_SLICE_MAX );
+    h->param.i_threads = x264_clip3( h->param.i_threads, 1, X264_THREADS_MAX );
     h->param.i_threads = X264_MIN( h->param.i_threads, (h->param.i_height + 15) / 16 );
 #if !(HAVE_PTHREAD)
     if( h->param.i_threads > 1 )
@@ -536,7 +536,13 @@
 
 
     /* Init x264_t */
+    /* Create enough place for NAL-packets */
+    
+    /* We need 3 NALs for possible SPS, PPS and SEI and at least 1 for a slice */
+    h->out.i_nal_bufsize = 4;
     h->out.i_nal = 0;
+    h->out.nal = x264_malloc( h->out.i_nal_bufsize * sizeof( x264_nal_t ) );
+    
     h->out.i_bitstream = X264_MAX( 1000000, h->param.i_width * h->param.i_height * 1.7
         * ( h->param.rc.b_cbr ? pow( 0.5, h->param.rc.i_qp_min )
           : pow( 0.5, h->param.rc.i_qp_constant ) * X264_MAX( 1, h->param.rc.f_ip_factor )));
@@ -682,6 +688,7 @@
 {
     /* init bitstream context */
     h->out.i_nal = 0;
+    
     bs_init( &h->out.bs, h->out.p_bitstream, h->out.i_bitstream );
 
     /* Put SPS and PPS */
@@ -921,9 +928,8 @@
     int i_skip;
     int mb_xy;
     int i;
-
-    /* init stats */
-    memset( &h->stat.frame, 0, sizeof(h->stat.frame) );
+    int i_pos_max;
+    
 
     /* Slice */
     x264_nal_start( h, h->i_nal_type, h->i_nal_ref_idc );
@@ -941,8 +947,14 @@
     }
     h->mb.i_last_qp = h->sh.i_qp;
     h->mb.i_last_dqp = 0;
-
-    for( mb_xy = h->sh.i_first_mb, i_skip = 0; mb_xy < h->sh.i_last_mb; mb_xy++ )
+    
+    mb_xy = h->sh.i_first_mb;
+    i_skip = 0;
+    i_pos_max = bs_pos(&h->out.bs) + h->param.i_slice_max;
+    
+    while ( mb_xy < h->sh.i_last_mb 
+                && ( h->param.i_slice_max > 0 ? 
+                bs_pos( &h->out.bs  ) < i_pos_max : 1 ) )
     {
         const int i_mb_y = mb_xy / h->sps->i_mb_width;
         const int i_mb_x = mb_xy % h->sps->i_mb_width;
@@ -1031,6 +1043,8 @@
 
         if( h->mb.b_variable_qp )
             x264_ratecontrol_mb(h, bs_pos(&h->out.bs) - mb_spos);
+        
+        mb_xy++;
     }
 
     if( h->param.b_cabac )
@@ -1057,15 +1071,44 @@
     x264_nal_end( h );
 
     /* Compute misc bits */
-    h->stat.frame.i_misc_bits = bs_pos( &h->out.bs )
+    h->stat.frame.i_misc_bits += bs_pos( &h->out.bs )
                               + NALU_OVERHEAD * 8
                               - h->stat.frame.i_itex_bits
                               - h->stat.frame.i_ptex_bits
                               - h->stat.frame.i_hdr_bits;
 
+    /* Needed to check if there is another slice possible in the current frame */
+    h->sh.i_first_mb = mb_xy;
+    
     return 0;
 }
 
+static int x264_slice_write_thread( x264_t *h )
+{
+    /* Not needed atm, but for possible future usage */
+    int i_return = 0;
+    
+    /* init stats */
+    memset( &h->stat.frame, 0, sizeof(h->stat.frame) );
+    
+    /* Run as long as the last MB isn't reached */
+    while( !i_return && h->sh.i_first_mb != h->sh.i_last_mb )
+    {
+        i_return = x264_slice_write( h );
+        
+        /* More nal's when needed */
+        if( h->out.i_nal >= h->out.i_nal_bufsize )
+        {
+            /* This should work better than just adding a constant number each time */
+            h->out.i_nal_bufsize *= 2;
+            h->out.nal = x264_realloc( h->out.nal, h->out.i_nal_bufsize * sizeof( x264_nal_t ) );
+
+        }
+    }
+    
+    return i_return;
+}
+
 static inline int x264_slices_write( x264_t *h )
 {
     int i_frame_size;
@@ -1077,12 +1120,11 @@
 
     if( h->param.i_threads == 1 )
     {
-        x264_slice_write( h );
+        x264_slice_write_thread( h );
         i_frame_size = h->out.nal[h->out.i_nal-1].i_payload;
     }
     else
-    {
-        int i_nal = h->out.i_nal;
+    {        
         int i_bs_size = h->out.i_bitstream / h->param.i_threads;
         int i;
         /* duplicate contexts */
@@ -1092,42 +1134,60 @@
             if( i > 0 )
             {
                 memcpy( t, h, sizeof(x264_t) );
+                
+                /* Create a place for the NALs to go for the moment */
+                
+                /* Sane value */
+                t->out.i_nal_bufsize = 4;
+                t->out.i_nal = 0;
+                t->out.nal = x264_malloc( h->out.i_nal_bufsize * sizeof( x264_nal_t ) );
+                
                 t->out.p_bitstream += i*i_bs_size;
                 bs_init( &t->out.bs, t->out.p_bitstream, i_bs_size );
             }
             t->sh.i_first_mb = (i    * h->sps->i_mb_height / h->param.i_threads) * h->sps->i_mb_width;
             t->sh.i_last_mb = ((i+1) * h->sps->i_mb_height / h->param.i_threads) * h->sps->i_mb_width;
-            t->out.i_nal = i_nal + i;
         }
 
         /* dispatch */
 #if HAVE_PTHREAD
         {
-            pthread_t handles[X264_SLICE_MAX];
+            pthread_t handles[X264_THREADS_MAX];
             void *status;
             for( i = 0; i < h->param.i_threads; i++ )
-                pthread_create( &handles[i], NULL, (void*)x264_slice_write, (void*)h->thread[i] );
+                pthread_create( &handles[i], NULL, (void*)x264_slice_write_thread, (void*)h->thread[i] );
             for( i = 0; i < h->param.i_threads; i++ )
                 pthread_join( handles[i], &status );
         }
 #else
         for( i = 0; i < h->param.i_threads; i++ )
-            x264_slice_write( h->thread[i] );
+            x264_slice_write_thread( h->thread[i] );
 #endif
 
         /* merge contexts */
-        i_frame_size = h->out.nal[i_nal].i_payload;
+        i_frame_size = h->out.nal[h->out.i_nal].i_payload;
         for( i = 1; i < h->param.i_threads; i++ )
         {
             int j;
             x264_t *t = h->thread[i];
-            h->out.nal[i_nal+i] = t->out.nal[i_nal+i];
-            i_frame_size += t->out.nal[i_nal+i].i_payload;
+            
+            /* Copy the thread's NAL units back into the main thread. */
+            if ( h->out.i_nal + t->out.i_nal >= h->out.i_nal_bufsize )
+            {
+                h->out.i_nal_bufsize = h->out.i_nal + t->out.i_nal + 1;
+                h->out.nal = x264_realloc( h->out.nal, h->out.i_nal_bufsize * sizeof( x264_nal_t ) );
+            }
+            memcpy( &h->out.nal[h->out.i_nal], t->out.nal, t->out.i_nal * sizeof( x264_nal_t ) );
+            h->out.i_nal += t->out.i_nal;
+            
+            /* Free the resources */
+            x264_free( t->out.nal );
+            
+            i_frame_size += t->out.nal[t->out.i_nal+i].i_payload;
             // all entries in stat.frame are ints
             for( j = 0; j < sizeof(h->stat.frame) / sizeof(int); j++ )
                 ((int*)&h->stat.frame)[j] += ((int*)&t->stat.frame)[j];
         }
-        h->out.i_nal = i_nal + h->param.i_threads;
     }
 
 #if VISUALIZE
@@ -1779,6 +1839,7 @@
     x264_free( h->out.p_bitstream );
     for( i = 1; i < h->param.i_threads; i++ )
         x264_free( h->thread[i] );
+    x264_free( h->out.nal );
     x264_free( h );
 }
 
diff -ur x264.r439/x264.c x264/x264.c
--- x264.r439/x264.c	2006-02-20 20:34:01.000000000 +0100
+++ x264/x264.c	2006-02-20 23:45:01.000000000 +0100
@@ -238,6 +238,10 @@
              "                                  where <option> is either\n"
              "                                      q=<integer> (force QP)\n"
              "                                  or  b=<float> (bitrate multiplier)\n"
+             "      --max-slicesize <integer>  Set the (estimated) maximum slicesize in bytes\n"
+             "                                  Disabled when set to 0 (=default)\n"
+             "                                  The slicesize will always be bigger as\n"
+             "                                      +/- 400 bytes (= slice header + 1 MB)\n"
              "\n"
              "Analysis:\n"
              "\n"
@@ -488,6 +492,7 @@
 #define OPT_NO_FAST_PSKIP 317
 #define OPT_BIME 318
 #define OPT_NR 319
+#define OPT_SLICE_MAX 320
 
         static struct option long_options[] =
         {
@@ -544,6 +549,7 @@
             { "cplxblur",required_argument, NULL, OPT_CPLXBLUR },
             { "zones",   required_argument, NULL, OPT_ZONES },
             { "threads", required_argument, NULL, OPT_THREADS },
+            { "max-slicesize", required_argument, NULL, OPT_SLICE_MAX },
             { "no-psnr", no_argument,       NULL, OPT_NOPSNR },
             { "quiet",   no_argument,       NULL, OPT_QUIET },
             { "verbose", no_argument,       NULL, 'v' },
@@ -830,6 +836,9 @@
             case OPT_THREADS:
                 param->i_threads = atoi(optarg);
                 break;
+            case OPT_SLICE_MAX:
+                param->i_slice_max = atoi(optarg);
+                break;
             case OPT_NOPSNR:
                 param->analyse.b_psnr = 0;
                 break;
diff -ur x264.r439/x264.h x264/x264.h
--- x264.r439/x264.h	2006-02-20 20:34:01.000000000 +0100
+++ x264/x264.h	2006-02-20 20:51:13.000000000 +0100
@@ -149,6 +149,8 @@
     int         i_fps_num;
     int         i_fps_den;
 
+    int         i_slice_max; /* maximum slice size in bytes, not used if 0 */
+
     /* Bitstream parameters */
     int         i_frame_reference;  /* Maximum number of reference frames */
     int         i_keyint_max;       /* Force an IDR keyframe at this interval */
